<?php

namespace App\Controller\OgBoot\OgLive;

use App\Controller\OgBoot\AbstractOgBootController;
use App\Entity\OgLive;
use App\Model\OgLiveStatus;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class SyncAction extends AbstractOgBootController
{
    const string OG_BOOT_DIRECTORY = '/opt/opengnsys/ogboot/tftpboot//';
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(): JsonResponse
    {
        $content = $this->createRequest('GET', 'http://'.$this->ogBootApiUrl . '/ogboot/v1/oglives');

        foreach ($content['message']['installed_ogLives'] as $ogLive) {
            $ogLiveEntity = $this->entityManager->getRepository(OgLive::class)->findOneBy(['checksum' => $ogLive['id']]);

            if (!$ogLiveEntity) {
                $ogLiveEntity = $this->entityManager->getRepository(OgLive::class)->findOneBy(['filename' => str_replace(self::OG_BOOT_DIRECTORY, '', $ogLive['directory'])]);

                if (!$ogLiveEntity) {
                    $ogLiveEntity = new OgLive();
                }
            }
            $this->extracted($ogLiveEntity, $ogLive);
            $this->entityManager->persist($ogLiveEntity);
        }
        $this->entityManager->flush();

        if (isset($content['message']['default_oglive'])) {
            $this->serDefaultOgLive($content['message']['default_oglive']);
        }

        return new JsonResponse(data: $content, status: Response::HTTP_OK);
    }

    /**
     * @param OgLive|null $ogLiveEntity
     * @param mixed $ogLive
     * @return void
     * @throws \Exception
     */
    private function extracted(OgLive $ogLiveEntity, mixed $ogLive): void
    {
        $ogLiveEntity->setName($this->symflipyOgLiveFilenameService->__invoke(str_replace(self::OG_BOOT_DIRECTORY, '', $ogLive['directory'])));
        $ogLiveEntity->setDate(new \DateTime($this->extractOgLiveFilenameDateService->__invoke(str_replace(self::OG_BOOT_DIRECTORY, '', $ogLive['directory']))));
        $ogLiveEntity->setInstalled(true);
        $ogLiveEntity->setArchitecture($ogLive['architecture']);
        $ogLiveEntity->setDistribution($ogLive['distribution']);
        $ogLiveEntity->setFilename(str_replace(self::OG_BOOT_DIRECTORY, '', $ogLive['directory']));
        $ogLiveEntity->setKernel($ogLive['kernel']);
        $ogLiveEntity->setRevision($ogLive['revision']);
        $ogLiveEntity->setDirectory($ogLive['directory']);
        $ogLiveEntity->setChecksum($ogLive['id']);
        $ogLiveEntity->setStatus(OgLiveStatus::ACTIVE);
    }

    private function serDefaultOgLive(string $defaultOgLive): void
    {
        $ogLiveEntity = $this->entityManager->getRepository(OgLive::class)->findOneBy(['filename' => $defaultOgLive]);

        if (!$ogLiveEntity) {
            return;
        }

        $ogLiveEntity->setIsDefault(true);
        $this->entityManager->persist($ogLiveEntity);
        $this->entityManager->flush();
    }
}