<?php

namespace App\Entity;

use App\Repository\ImageImageRepositoryRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;

#[ORM\Entity(repositoryClass: ImageImageRepositoryRepository::class)]
#[ORM\UniqueConstraint(name: 'UNIQ_IDENTIFIER_IMAGE_REPOSITORY', columns: ['image_id', 'repository_id'])]
#[UniqueEntity(fields: ['image', 'repository'], message: 'This image is already associated with this repository')]
class ImageImageRepository extends AbstractEntity
{
    #[ORM\ManyToOne(targetEntity: Image::class, cascade: ['persist'], inversedBy: 'imageImageRepositories')]
    #[ORM\JoinColumn(nullable: false)]
    private ?Image $image = null;

    #[ORM\ManyToOne(targetEntity: ImageRepository::class, cascade: ['persist'], inversedBy: 'imageImageRepositories')]
    #[ORM\JoinColumn(nullable: false)]
    private ?ImageRepository $repository = null;

    #[ORM\Column(length: 255)]
    private ?string $status = null;

    #[ORM\Column(nullable: true)]
    private ?bool $created = null;

    #[ORM\Column(length: 255, nullable: true)]
    private ?string $imageFullsum = null;

    public function getImage(): ?Image
    {
        return $this->image;
    }

    public function setImage(?Image $image): static
    {
        $this->image = $image;

        return $this;
    }

    public function getRepository(): ?ImageRepository
    {
        return $this->repository;
    }

    public function setRepository(?ImageRepository $repository): static
    {
        $this->repository = $repository;

        return $this;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(string $status): static
    {
        $this->status = $status;

        return $this;
    }

    public function getCreated(): ?bool
    {
        return $this->created;
    }

    public function setCreated(?bool $created): static
    {
        $this->created = $created;

        return $this;
    }

    public function getImageFullsum(): ?string
    {
        return $this->imageFullsum;
    }

    public function setImageFullsum(?string $imageFullsum): static
    {
        $this->imageFullsum = $imageFullsum;

        return $this;
    }
}
